@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2024 alx@fastestcode.org
+ * Copyright (C) 2012-2025 alx@fastestcode.org
  * This software is distributed under the terms of the MIT license.
  * See the included LICENSE file for further information.
  */
@@ -152,6 +152,7 @@ static struct browser_data* create_browser(const struct app_resources *res)
 	XGCValues gc_values;
 	static Pixmap wmicon=0;
 	static Pixmap wmicon_mask=0;
+	Pixmap bg_pixmap;
 	XtCallbackRec path_change_cbr[] = {
 		{ path_change_cb, NULL }, { NULL, NULL }
 	};
@@ -183,11 +184,6 @@ static struct browser_data* create_browser(const struct app_resources *res)
 	bd->wmain=XmVaCreateMainWindow(bd->wshell,"main",NULL);
 	create_browser_menubar(bd);
 
-	XtVaGetValues(bd->wmain,XmNcolormap,&colormap,
-		XmNbackground,&bd->bg_pixel,NULL);
-	XmGetColors(XtScreen(bd->wshell),colormap,bd->bg_pixel,
-		&bd->fg_pixel,&bd->ts_pixel,&bd->bs_pixel,&bd->sbg_pixel);
-			
 	XtVaGetValues(XmGetXmDisplay(app_inst.display),
 		XmNenableThinThickness,&line_width,NULL);
 	line_width=(line_width ? 1:2);
@@ -228,8 +224,8 @@ static struct browser_data* create_browser(const struct app_resources *res)
 	XtAddCallback(bd->wvscroll,XmNvalueChangedCallback,
 		scroll_cb,(XtPointer)bd);
 	
-	bd->wview=XmVaCreateManagedDrawingArea(wview_scrl,"view",
-		XmNbackground,bd->bg_pixel,XmNuserData,(XtPointer)bd,NULL);
+	bd->wview=XmVaCreateManagedDrawingArea(wview_scrl,
+		"view", XmNuserData, (XtPointer)bd, NULL);
 	XtAddEventHandler(bd->wview,StructureNotifyMask,
 		False,view_map_cb,(XtPointer)bd);
 	if(!view_tt){
@@ -249,7 +245,17 @@ static struct browser_data* create_browser(const struct app_resources *res)
 	XtOverrideTranslations(bd->wview,view_tt);
 	XtVaSetValues(wview_scrl,XmNverticalScrollBar,bd->wvscroll,
 		XmNworkWindow,bd->wview,NULL);
-		
+
+	XtVaGetValues(bd->wview, XmNcolormap, &colormap,
+		XmNbackgroundPixmap, &bg_pixmap,
+		XmNbackground, &bd->bg_pixel, NULL);
+	XmGetColors(XtScreen(bd->wshell), colormap, bd->bg_pixel,
+		&bd->fg_pixel, &bd->ts_pixel, &bd->bs_pixel, &bd->sbg_pixel);
+
+	/* see if the user has specified a background pixmap, in which
+	 * case we can't do optimized scrolling */
+	if(bg_pixmap != XmUNSPECIFIED_PIXMAP) bd->has_bg_pixmap = True;
+			
 	wmsgbar=XmVaCreateManagedFrame(bd->wmain,"messageArea",
 		XmNshadowType,XmSHADOW_OUT,
 		XmNshadowThickness, 1,NULL);
@@ -703,9 +709,9 @@ static int read_directory(struct browser_data *bd)
 		if(S_ISDIR(st.st_mode)) {
 			long i;
 			
-			if(!strcmp(dir_ent->d_name, "..")) continue;
-
-			if(!bd->show_dot_files && dir_ent->d_name[0] == '.') continue;
+			if(!bd->show_dot_files &&
+				(dir_ent->d_name[0] == '.') &&
+				strcmp(dir_ent->d_name, "..")) continue;
 			
 			for(i = 0; i < bd->nsubdirs; i++) {
 				if(!strcmp(bd->subdirs[i], dir_ent->d_name)) break;
@@ -2020,7 +2026,7 @@ static void set_vscroll(struct browser_data *bd, int new_offset)
 
 	bd->yoffset = new_offset;
 	
-	if(abs(delta) >= bd->view_height) {
+	if((abs(delta) >= bd->view_height) || bd->has_bg_pixmap) {
 		XClearArea(app_inst.display, view, 0, 0,
 			bd->view_width, bd->view_height, True);
 		return;
@@ -2110,17 +2116,19 @@ static void update_shell_title(struct browser_data *bd)
 	if(bd->path){
 		char *buffer;
 		char *dir;
-		dir=strrchr(bd->path,'/');
+		dir = strrchr(bd->path, '/');
+
 		if(!dir)
-			dir=bd->path;
-		else if(dir[1]!='\0') dir++;
-		buffer=malloc(strlen(bd->title)+strlen(dir)+4);
-		sprintf(buffer,"%s - %s",dir,bd->title);
-		XtVaSetValues(bd->wshell,XmNtitle,buffer,XmNiconName,buffer,NULL);
+			dir = bd->path;
+		else if(dir[1] != '\0') dir++;
+
+		buffer = malloc(strlen(bd->title) + strlen(dir) + 4);
+		sprintf(buffer, "%s - %s", dir, bd->title);
+
+		set_shell_title(bd->wshell, buffer, buffer);
 		free(buffer);
-	}else{
-		XtVaSetValues(bd->wshell,XmNtitle,bd->title,
-			XmNiconName,bd->title,NULL);
+	} else {
+		set_shell_title(bd->wshell, bd->title, bd->title);
 	}
 }
 
@@ -2897,21 +2905,23 @@ static void dirlist_cb(Widget w, XtPointer client, XtPointer call)
 	unsigned int pos = cbs->item_position - 1;
 	char *new_path;
 	
-	if( (cbs->reason != XmCR_DEFAULT_ACTION) ||
-		(!strcmp(bd->subdirs[pos], "..") && !strcmp(bd->path, "/")) ) return;
-
-	new_path = malloc(strlen(bd->path) +
-		strlen(bd->subdirs[pos]) + 2);
-	if(!new_path) {
-		message_box(app_inst.session_shell, MB_ERROR, BASE_NAME,
-				nlstr(APP_MSGSET,SID_ENORES,
-				"Not enough resources available for this task."));
-		return;
-	}
-	sprintf(new_path, "%s/%s", bd->path, bd->subdirs[pos]);
-	load_path(bd, new_path);
-	free(new_path);
+	if(cbs->reason != XmCR_DEFAULT_ACTION) return;
 	
+	if(!strcmp(bd->subdirs[pos], "..")) {
+		dir_up(bd);
+	} else {
+		new_path = malloc(strlen(bd->path) +
+			strlen(bd->subdirs[pos]) + 2);
+		if(!new_path) {
+			message_box(app_inst.session_shell, MB_ERROR, BASE_NAME,
+					nlstr(APP_MSGSET,SID_ENORES,
+					"Not enough resources available for this task."));
+			return;
+		}
+		sprintf(new_path, "%s/%s", bd->path, bd->subdirs[pos]);
+		load_path(bd, new_path);
+		free(new_path);
+	}
 	XmProcessTraversal(bd->wdirlist, XmTRAVERSE_CURRENT);
 }
 
