@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2024 alx@fastestcode.org
+ * Copyright (C) 2012-2025 alx@fastestcode.org
  * This software is distributed under the terms of the MIT license.
  * See the included LICENSE file for further information.
  */
@@ -275,13 +275,14 @@ static int copy_file(struct proc_data *pd, const char *src, const char *dest)
 	ssize_t rw;
 	int res=0;
 
-	if(!stat(dest,&st) && (pd->proc!=FPROC_MOVE)){
+	if(!stat(dest ,&st) && (pd->proc != FPROC_MOVE)){
 		enum response_code rc;
-		rc=ask_replace(pd,dest);
-		if(rc!=RC_CONTINUE){
-			if(rc==RC_QUIT) pd->cancelled=True;
+		rc = ask_replace(pd,dest);
+		if(rc != RC_CONTINUE){
+			if(rc == RC_QUIT) pd->cancelled = True;
 			return 0;
 		}
+		if(unlink(dest)) return errno;
 	}
 	if(stat(src,&st)) return errno;
 	fin=open(src,O_RDONLY);
