@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2024 alx@fastestcode.org
+ * Copyright (C) 2012-2025 alx@fastestcode.org
  * This software is distributed under the terms of the MIT license.
  * See the included LICENSE file for further information.
  */
@@ -142,8 +142,6 @@ static struct viewer_data* create_viewer(const struct app_resources *res)
 	int width=0, height=0;
 	XGCValues gc_values;
 	struct viewer_data *vd=viewers;
-	Colormap cmap;
-	Pixel bg_pixel, tmp_pixel;
 	static Pixmap wmicon=0;
 	static Pixmap wmicon_mask=0;
 	
@@ -185,11 +183,8 @@ static struct viewer_data* create_viewer(const struct app_resources *res)
 	create_viewer_menubar(vd);
 	create_viewer_toolbar(vd);
 
-	XtVaGetValues(vd->wmain,XmNcolormap,&cmap,XmNbackground,&tmp_pixel,NULL);
-	XmGetColors(XtScreen(vd->wshell),cmap,
-		tmp_pixel,&tmp_pixel,&tmp_pixel,&tmp_pixel,&bg_pixel);
-	vd->wview=XmVaCreateManagedDrawingArea(
-		vd->wmain,"view",XmNbackground,bg_pixel,XmNuserData,vd,NULL);
+	vd->wview=XmVaCreateManagedDrawingArea(vd->wmain,
+		"view", XmNuserData, vd, NULL);
 
 	/* create mouse translations for continguous panning and 
 	 * add keyboard interaction actions if not done yet */
@@ -696,7 +691,10 @@ static void load_next_file(struct viewer_data *vd, Bool forward)
 		strlen(vd->dir_files[vd->dir_cur_file])+2);
 	sprintf(new_file_name,"%s/%s",vd->dir_name,
 		vd->dir_files[vd->dir_cur_file]);
-	load_image(vd, new_file_name, NULL);
+	
+	if(!vd->file_name || strcmp(vd->file_name, new_file_name))
+		load_image(vd, new_file_name, NULL);
+
 	free(new_file_name);
 }
 
@@ -937,6 +935,8 @@ static void* dir_reader_thread(void *arg)
 	int ret_code=0;
 	char **files=NULL;
 	unsigned long nfiles=0;
+	char *tmp_name = NULL;
+	size_t tmp_name_len = 0;
 
 	dbg_assert(vd->dir_name);
 	
@@ -959,6 +959,8 @@ static void* dir_reader_thread(void *arg)
 	}
 	
 	while((dir_ent=readdir(dir))){
+		size_t tmp_len;
+		
 		if(vd->state & DSF_CANCEL) {
 			closedir(dir);
 			goto exit_thread;
@@ -979,12 +981,27 @@ static void* dir_reader_thread(void *arg)
 			files=new_buf;
 		}
 		
-		if(!img_ident(dir_ent->d_name, NULL, NULL)){
+		tmp_len = strlen(vd->dir_name) + strlen(dir_ent->d_name) + 2;
+		if(tmp_len > tmp_name_len) {
+			char *new_buf = realloc(tmp_name, tmp_len);
+			if(!new_buf) {
+				ret_code = errno;
+				closedir(dir);
+				free(tmp_name);
+				goto exit_thread;
+			}
+			tmp_name = new_buf;
+			tmp_name_len = tmp_len;
+		}
+		sprintf(tmp_name, "%s/%s", vd->dir_name, dir_ent->d_name);
+
+		if(!img_ident(tmp_name, NULL, NULL)){
 			files[nfiles] = strdup(dir_ent->d_name);
 		} else continue;
 		
 		nfiles++;
 	}
+	free(tmp_name);
 	closedir(dir);
 	
 	if(nfiles){
@@ -1165,21 +1182,24 @@ static void compute_image_dimensions(struct viewer_data *vd,
  */
 static void update_shell_title(struct viewer_data *vd)
 {
-	if(vd->state&ISF_OPENED){
+	if(vd->state & ISF_OPENED){
 		char *buffer;
 		char *ftitle;
-		ftitle=strrchr(vd->file_name,'/');
+		ftitle = strrchr(vd->file_name,'/');
+		
 		if(!ftitle)
-			ftitle=vd->file_name;
+			ftitle = vd->file_name;
 		else
 			ftitle++;
-		buffer=malloc(strlen(vd->title)+strlen(ftitle)+4);
-		sprintf(buffer,"%s - %s",ftitle,vd->title);
-		XtVaSetValues(vd->wshell,XmNtitle,buffer,XmNiconName,buffer,NULL);
+		
+		buffer = malloc(strlen(vd->title) + strlen(ftitle) + 4);
+		sprintf(buffer,"%s - %s", ftitle, vd->title);
+		
+		set_shell_title(vd->wshell, buffer, buffer);
+		
 		free(buffer);
 	}else{
-		XtVaSetValues(vd->wshell,XmNtitle,vd->title,
-			XmNiconName,vd->title,NULL);
+		set_shell_title(vd->wshell, vd->title, vd->title);
 	}
 }
 
@@ -2000,7 +2020,26 @@ static void rotate_left_cb(Widget w, XtPointer client, XtPointer call)
 static void rotate_reset_cb(Widget w, XtPointer client, XtPointer call)
 {
 	struct viewer_data *vd=(struct viewer_data*)client;
-	vd->tform=0;
+	
+	if(vd->tform & IMGT_ROTATE) {
+		Dimension vw, vh;
+		int iw, ih;
+		float vpc = 0;
+		float hpc = 0;
+		
+		XtVaGetValues(vd->wview, XmNwidth, &vw, XmNheight, &vh, NULL);
+		compute_image_dimensions(vd, vd->zoom, vd->tform, &iw, &ih);
+		
+		if(vw < iw) hpc = (float)abs(vd->xoff) / (iw - vw);
+		if(vh < ih) vpc = (float)abs(vd->yoff) / (ih - vh);
+	
+		compute_image_dimensions(vd, vd->zoom, 0, &iw, &ih);
+		
+		if(vw < iw) vd->xoff = -((iw - vw) * hpc); else vd->xoff = 0;
+		if(vh < ih) vd->yoff = -((ih - vh) * vpc); else vd->yoff = 0;
+	}
+	
+	vd->tform = 0;
 	update_back_buffer(vd);
 	redraw_view(vd,True);
 }
